@@ -1,4 +1,5 @@
 #![warn(clippy::pedantic)]
+#![allow(clippy::disallowed_names, clippy::type_complexity)]
 
 use typed_builder::TypedBuilder;
 
@@ -66,6 +67,49 @@ fn test_generics() {
     assert!(Foo::builder().x(1).y(2).build() == Foo { x: 1, y: 2 });
 }
 
+#[test]
+fn test_2d_const_generics() {
+    #[derive(PartialEq, TypedBuilder)]
+    struct Foo<const NUM_COLS: usize, const NUM_ROWS: usize> {
+        data: [[u32; NUM_ROWS]; NUM_COLS],
+    }
+    assert!(Foo::builder().data([[]]).build() == Foo { data: [[]] });
+}
+
+#[test]
+fn test_multiple_const_generics() {
+    #[derive(PartialEq, TypedBuilder)]
+    struct Foo<const A: usize, const B: usize, const C: usize> {
+        data: [u32; A],
+        data2: [u32; B],
+        data3: [u32; C],
+    }
+    assert!(
+        Foo::builder().data([1]).data2([2, 3]).data3([]).build()
+            == Foo {
+                data: [1],
+                data2: [2, 3],
+                data3: []
+            }
+    );
+}
+
+#[test]
+fn test_const_generics_with_other_generics() {
+    #[derive(PartialEq, TypedBuilder)]
+    struct Foo<const A: usize, B> {
+        data: [B; A],
+        data2: [B; 3],
+    }
+    assert!(
+        Foo::builder().data([3]).data2([0, 1, 2]).build()
+            == Foo {
+                data: [3],
+                data2: [0, 1, 2]
+            }
+    );
+}
+
 #[test]
 fn test_into() {
     #[derive(PartialEq, TypedBuilder)]
@@ -99,6 +143,30 @@ fn test_into_with_strip_option() {
     assert!(Foo::builder().x(1_u8).build() == Foo { x: Some(1) });
 }
 
+#[test]
+fn test_strip_option_with_fallback() {
+    #[derive(PartialEq, TypedBuilder)]
+    struct Foo {
+        #[builder(setter(strip_option(fallback = x_opt)))]
+        x: Option<i32>,
+    }
+
+    assert!(Foo::builder().x(1).build() == Foo { x: Some(1) });
+    assert!(Foo::builder().x_opt(Some(1)).build() == Foo { x: Some(1) });
+}
+
+#[test]
+fn test_into_with_strip_option_with_fallback() {
+    #[derive(PartialEq, TypedBuilder)]
+    struct Foo {
+        #[builder(setter(into, strip_option(fallback = x_opt)))]
+        x: Option<i32>,
+    }
+
+    assert!(Foo::builder().x(1_u8).build() == Foo { x: Some(1) });
+    assert!(Foo::builder().x_opt(Some(1)).build() == Foo { x: Some(1) });
+}
+
 #[test]
 fn test_strip_bool() {
     #[derive(PartialEq, TypedBuilder)]
@@ -111,6 +179,19 @@ fn test_strip_bool() {
     assert!(Foo::builder().build() == Foo { x: false });
 }
 
+#[test]
+fn test_strip_bool_with_fallback() {
+    #[derive(PartialEq, TypedBuilder)]
+    struct Foo {
+        #[builder(setter(into, strip_bool(fallback = x_bool)))]
+        x: bool,
+    }
+
+    assert!(Foo::builder().x().build() == Foo { x: true });
+    assert!(Foo::builder().x_bool(false).build() == Foo { x: false });
+    assert!(Foo::builder().build() == Foo { x: false });
+}
+
 #[test]
 fn test_default() {
     #[derive(PartialEq, TypedBuilder)]
@@ -504,7 +585,7 @@ fn test_clone_builder_with_generics() {
     let semi_built2 = Foo::builder().x("four");
 
     assert!(semi_built2.clone().y(5).build() == Foo { x: "four", y: 5 });
-    assert!(semi_built2.clone().y(6).build() == Foo { x: "four", y: 6 });
+    assert!(semi_built2.y(6).build() == Foo { x: "four", y: 6 });
 
     // Just to make sure it can build with generic bounds
     #[allow(dead_code)]
@@ -542,6 +623,46 @@ fn test_builder_on_struct_with_keywords() {
     );
 }
 
+#[test]
+fn test_builder_on_struct_with_keywords_prefix_suffix() {
+    #[allow(non_camel_case_types)]
+    #[derive(PartialEq, TypedBuilder)]
+    #[builder(field_defaults(setter(prefix = "set_", suffix = "_value")))]
+    struct r#struct {
+        r#fn: u32,
+        #[builder(default, setter(strip_option))]
+        r#type: Option<u32>,
+        #[builder(default = Some(()), setter(skip))]
+        r#enum: Option<()>,
+        #[builder(setter(into))]
+        r#union: String,
+    }
+
+    assert!(
+        r#struct::builder().r#set_fn_value(1).r#set_union_value("two").build()
+            == r#struct {
+                r#fn: 1,
+                r#type: None,
+                r#enum: Some(()),
+                r#union: "two".to_owned(),
+            }
+    );
+}
+
+#[test]
+fn test_unsized_generic_params() {
+    use std::marker::PhantomData;
+
+    #[derive(TypedBuilder)]
+    struct GenericStructure<K, V>
+    where
+        K: ?Sized,
+    {
+        key: PhantomData<K>,
+        value: PhantomData<V>,
+    }
+}
+
 #[test]
 fn test_field_setter_transform() {
     #[derive(PartialEq)]
@@ -649,6 +770,18 @@ fn test_into_set_generic_impl_from() {
     assert_eq!(bar, Bar { value: 42 });
 }
 
+#[test]
+fn test_into_angle_bracket_type() {
+    #[derive(Debug, PartialEq, TypedBuilder)]
+    #[builder(build_method(into = std::sync::Arc<Foo>))]
+    struct Foo {
+        value: i32,
+    }
+
+    let foo: std::sync::Arc<Foo> = Foo::builder().value(42).build();
+    assert_eq!(*foo, Foo { value: 42 });
+}
+
 #[test]
 fn test_into_set_generic_impl_into() {
     #[derive(TypedBuilder)]
@@ -662,12 +795,199 @@ fn test_into_set_generic_impl_into() {
         value: i32,
     }
 
-    impl Into<Bar> for Foo {
-        fn into(self) -> Bar {
-            Bar { value: self.value }
+    impl From<Foo> for Bar {
+        fn from(val: Foo) -> Self {
+            Self { value: val.value }
         }
     }
 
     let bar: Bar = Foo::builder().value(42).build();
     assert_eq!(bar, Bar { value: 42 });
 }
+
+#[test]
+fn test_prefix() {
+    #[derive(Debug, PartialEq, TypedBuilder)]
+    #[builder(field_defaults(setter(prefix = "with_")))]
+    struct Foo {
+        x: i32,
+        y: i32,
+    }
+
+    let foo = Foo::builder().with_x(1).with_y(2).build();
+    assert_eq!(foo, Foo { x: 1, y: 2 });
+}
+
+#[test]
+fn test_suffix() {
+    #[derive(Debug, PartialEq, TypedBuilder)]
+    #[builder(field_defaults(setter(suffix = "_value")))]
+    struct Foo {
+        x: i32,
+        y: i32,
+    }
+
+    let foo = Foo::builder().x_value(1).y_value(2).build();
+    assert_eq!(foo, Foo { x: 1, y: 2 });
+}
+
+#[test]
+fn test_prefix_and_suffix() {
+    #[derive(Debug, PartialEq, TypedBuilder)]
+    #[builder(field_defaults(setter(prefix = "with_", suffix = "_value")))]
+    struct Foo {
+        x: i32,
+        y: i32,
+    }
+
+    let foo = Foo::builder().with_x_value(1).with_y_value(2).build();
+    assert_eq!(foo, Foo { x: 1, y: 2 });
+}
+
+#[test]
+fn test_issue_118() {
+    #[derive(TypedBuilder)]
+    #[builder(build_method(into=Bar))]
+    struct Foo<T> {
+        #[builder(default, setter(skip))]
+        #[allow(dead_code)]
+        foo: Option<T>,
+    }
+
+    struct Bar;
+
+    impl<T> From<Foo<T>> for Bar {
+        fn from(_value: Foo<T>) -> Self {
+            Self
+        }
+    }
+
+    let _ = Foo::<u32>::builder().build();
+}
+
+#[test]
+fn test_mutable_defaults() {
+    #[derive(TypedBuilder, PartialEq, Debug)]
+    struct Foo {
+        #[builder(default, mutable_during_default_resolution, setter(strip_option))]
+        x: Option<i32>,
+        #[builder(default = if let Some(x) = x.as_mut() {
+            *x *= 2;
+            *x
+        } else {
+            Default::default()
+        })]
+        y: i32,
+    }
+
+    let foo = Foo::builder().x(5).build();
+
+    assert_eq!(foo, Foo { x: Some(10), y: 10 });
+}
+
+#[test]
+fn test_preinitialized_fields() {
+    #[derive(Debug, PartialEq, TypedBuilder)]
+    struct Foo {
+        x: i32,
+        #[builder(via_mutators)]
+        y: i32,
+        #[builder(via_mutators = 2)]
+        z: i32,
+        #[builder(via_mutators(init = 2))]
+        w: i32,
+    }
+
+    let foo = Foo::builder().x(1).build();
+    assert_eq!(foo, Foo { x: 1, y: 0, z: 2, w: 2 });
+}
+
+#[test]
+fn test_mutators_item() {
+    #[derive(Debug, PartialEq, TypedBuilder)]
+    #[builder(mutators(
+        #[mutator(requires = [x])]
+        fn inc_x(self) {
+            self.x += 1;
+        }
+        #[mutator(requires = [x])]
+        fn inc_x_by(self, x: i32) {
+            self.x += x;
+        }
+        fn inc_preset(self) {
+            self.y += 1;
+            self.z += 1;
+            self.w += 1;
+        }
+        #[mutator(requires = [x])]
+        fn inc_y_by_x(self) {
+            self.y += self.x;
+        }
+    ))]
+    struct Foo {
+        x: i32,
+        #[builder(via_mutators)]
+        y: i32,
+        #[builder(via_mutators = 2)]
+        z: i32,
+        #[builder(via_mutators(init = 2))]
+        w: i32,
+    }
+
+    let foo = Foo::builder().x(1).inc_x().inc_preset().build();
+    assert_eq!(foo, Foo { x: 2, y: 1, z: 3, w: 3 });
+    let foo = Foo::builder().x(1).inc_x_by(4).inc_y_by_x().build();
+    assert_eq!(foo, Foo { x: 5, y: 5, z: 2, w: 2 });
+}
+
+#[test]
+fn test_mutators_field() {
+    #[derive(Debug, PartialEq, TypedBuilder)]
+    #[builder(mutators())]
+    struct Foo {
+        #[builder(mutators(
+            fn inc_x(self) {
+                self.x += 1;
+            }
+            #[mutator(requires = [y])]
+            fn inc_y_by_x(self) {
+                self.y += self.x;
+            }
+        ))]
+        x: i32,
+        #[builder(default)]
+        y: i32,
+        #[builder(via_mutators = 2, mutators(
+            fn inc_preset(self) {
+                self.z += 1;
+                self.w += 1;
+            }
+        ))]
+        z: i32,
+        #[builder(via_mutators(init = 2))]
+        w: i32,
+    }
+
+    let foo = Foo::builder().x(1).inc_x().inc_preset().build();
+    assert_eq!(foo, Foo { x: 2, y: 0, z: 3, w: 3 });
+    let foo = Foo::builder().x(1).y(1).inc_y_by_x().build();
+    assert_eq!(foo, Foo { x: 1, y: 2, z: 2, w: 2 });
+}
+
+#[test]
+fn test_mutators_for_generic_fields() {
+    use core::ops::AddAssign;
+
+    #[derive(Debug, PartialEq, TypedBuilder)]
+    struct Foo<S: Default + AddAssign, T: Default + AddAssign> {
+        #[builder(via_mutators, mutators(
+            fn x_plus(self, plus: S) {
+                self.x += plus;
+            }
+        ))]
+        x: S,
+        y: T,
+    }
+
+    assert_eq!(Foo::builder().x_plus(1).y(2).build(), Foo { x: 1, y: 2 });
+}
